# == Defined Type: postfix::ldap_map
#
# This type creates and manages a LDAP map file.
#
# === Parameters
# [**query_filter**]
#    The ldap query to run.
#
# [**result_attribute**]
#    The attribute whose value will to use for the map key.
#
# === Examples
#
# class { 'postfix':
#   ldap_base_dn = 'dc=example,dc=com',
#   ldap_bind_dn = 'uid=postfix,ou=services,dc=example,dc=com',
#   ldap_bind_pw = 'secretpassword',
# }
#
# Postfix::Ldap_map {
#   result_attribute => 'mail',
#   domain_map => 'mydestination',
# }
#
# postfix::ldap_map {
#   'mydestination':
#     query_filter => '(&(associatedDomain=%s))',
#     result_attribute => 'associateddomain';
#   'local_recipient_maps':
#     query_filter => '(&(|(mail=%s)(alias=%s))(|(objectclass=inetorgperson)(objectclass=groupofuniquenames)(objectclass=groupofurls)))';
#   'transport_maps':
#     query_filter => '(&(|(mailAlternateAddress=%s)(alias=%s)(mail=%s))(objectclass=inetorgperson))',
#     result_format => 'lmtp:unix:/var/lib/imap/socket/lmtp',
#     map_list => ['hash:/etc/postfix/transport'];
# }
#
# === Authors
#
# Brad2014 <brad2014@github.com>
#
# === Copyright
#
# Copyright 2016 Brad2014.
#


define postfix::ldap_map (
  $query_filter,
  $result_attribute,
  $base_dn = $::postfix::ldap_base_dn,
  $bind_dn = $::postfix::ldap_bind_dn,
  $bind_pw = $::postfix::ldap_bind_pw,
  $server = $::postfix::ldap_server,
  $port = $::postfix::ldap_port,
  $version = 3,
  $scope = 'sub',
  $domain_map = undef,
  $result_format = undef,
  $special_result_attribute = undef,
  $map_file = undef, # defaulted to "${ldap_dir}/${name}.cf", below
  $configuration_variable = $name,
  $map_list = [],
) {
  validate_string($base_dn)
  validate_string($bind_dn)
  validate_string($bind_pw)
  validate_string($server)
  validate_integer($port)
  validate_string($scope)
  validate_array($map_list)

  include ::postfix::params

  $ldap_dir = $::postfix::params::ldap_dir

  if is_string($domain_map) and $domain_map != $title {
    $domain_file = "${ldap_dir}/${domain_map}.cf"
    Postfix::Ldap_map[$domain_map] -> Postfix::Ldap_map[$title]
  }

  $file = pick( $map_file, "${ldap_dir}/${name}.cf" )

  ensure_resource( file, $ldap_dir, { ensure => directory, mode => '0700' } )

  file {
    $file: # CONTAINS SECRETS
      content => template('postfix/ldap_map.erb'),
      owner   => 'root',
      group   => 'root',
      mode    => '0400';
  }

  if $configuration_variable {
    $change_hash = { $configuration_variable => join(flatten([ "ldap:${file}", $map_list ]),', ') }
    postfix::configure {
      "ldap_map ${title}":
        changes => $change_hash;
    }
  }
}
