# == Defined Type: postfix::service
#
# This type creates and manages a service in master.cf.  See man page master.cf(5) for details.
#
# === Parameters
# [**command**]
#    The command to execute for this service. Required.
#
# [**service**]
#    The name of the service.  Defaults to `$title`.
#
# [**type**]
#    The type of the service.  Defaults to `unix`.
#
# [**private**]
#    Whether the service is private.  Values: '-', 'y', 'n'. Defaults to `-`.
#
# [**unpriv**]
#    Whether the service is unprivileged.  Values: '-', 'y', 'n'. Defaults to `-`.
#
# [**chroot**]
#    Whether the service runs under chroot.  Values: '-', 'y', 'n'. Defaults to `-`.
#
# [**wakeup**]
#    The wakeup time for the service.  Defaults to `-`.
#
# [**maxproc**]
#    The maximum number of the processes of this service.  Values: '-' (use default), 0 (unlimited), or an integer. Defaults to `-`.
#
# [**config_options**]
#    A hash of configuration variable names and values for this service.  Defaults to {} (i.e. no overrides).
#
# [**order**]
#    The relative location of this service in master.cf (does not affect operation). Defaults to 50.
#
# === Examples
#
# postfix::service {
#   'submission':
#     type => inet,
#     private => 'n',
#     chroot => 'n',
#     command => 'smtpd',
#     config_options => merge($submission_base_options, $submission_options),
#     order => 20;
# }
#
# === Authors
#
# Brad2014 <brad2014@github.com>
#
# === Copyright
#
# Copyright 2016 Brad2014.
#

define postfix::service (
  $command,
  $service = $title,
  $type = 'unix',
  $private = '-',
  $unpriv = '-',
  $chroot = '-',
  $wakeup = '-',
  $maxproc = '-',
  $config_options = {},
  $order = 50
) {
  include ::postfix::params

  validate_re("${type}", '^(inet|unix|fifo|pass)$')
  validate_re("${private}", '^[yn-]$')
  validate_re("${unpriv}", '^[yn-]$')
  validate_re("${chroot}", '^[yn-]$')
  validate_re("${wakeup}", '^([-]|[0-9]+[?]?)$')
  validate_string($command)
  validate_hash($config_options)

  $master_cf = $postfix::params::master_cf
  $option_block = prefix(sort(join_keys_to_values(delete_undef_values($config_options),'=')), "\n    -o ")

  concat::fragment {
    "${master_cf}_${title}":
      target  => $master_cf,
      content => join([
                  $service,
                  $type,
                  $private,
                  $unpriv,
                  $chroot,
                  $wakeup,
                  $maxproc,
                  "${command}${option_block}\n\n"
                ], ' '),
      order   => $order;
  }
}
